#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <gsl/gsl_rng.h>

/* https://www.wolframalpha.com/input/?i=lcm(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20)
 */
#define DEFAULT_STEPS 232792560ULL

/**
 *
 */
gsl_rng *new_random_generator(uint64_t seed)
{
  gsl_rng *random_generator;
  const gsl_rng_type *T;

  gsl_rng_env_setup();
  T = gsl_rng_default;
  random_generator = gsl_rng_alloc(T);
  gsl_rng_set(random_generator, seed);
  return random_generator;
}

/**
 *
 */
void free_random_generator(gsl_rng *random_generator)
{
  gsl_rng_free(random_generator);
}

/**
 *
 */
uint64_t compute_pi_montecarlo_hits(uint64_t steps, gsl_rng *random_generator)
{
  uint64_t i;
  uint64_t n_hits = 0;

  for (i = 0; i < steps; ++i) {
    double x = gsl_rng_uniform(random_generator);
    double y = gsl_rng_uniform(random_generator);
    double z = (x * x) + (y * y);
    if (z < 1.0) {
      ++n_hits;
    }
  }

  return n_hits;
}

/**
 *
 */
int main(int argc, char **argv)
{
  gsl_rng *random_generator;
  uint64_t steps;
  uint64_t hits;
  double pi_montecarlo;

  random_generator = new_random_generator(time(0));

  steps = DEFAULT_STEPS;

  hits = compute_pi_montecarlo_hits(steps, random_generator);

  pi_montecarlo = 4.0 * hits / steps;

  printf("Pi: %.8f\n", pi_montecarlo);

  free_random_generator(random_generator);
  return 0;
}
